#include "testlib.h"
#include <map>
#include <set>

#define verdict(x) (x == -1 ? _fail : _wa)

std::map<std::string, int> nouns, verbs, adjs;

bool proceed_sentence(std::string const &sentence, int &words, int &score, std::set<std::string> &occured_words, int const test) {
    int pos = 0, length = (int)sentence.length();
    while (pos != length && sentence[pos] == ' ') {
        ++pos;
    }
    if (pos == length) {
        return false;
    }
    
    if (!isupper(sentence[pos])) {
        return false;
    }
    if (!islower(sentence[length - 1])) {
        return false;
    }
    
    std::string word(1, tolower(sentence[pos]));
    bool is_word = true, noun_occured = false, last_adjective = false, verb_occured = false;
    
    for (++pos; pos <= length; ++pos) {
        if (pos != length && islower(sentence[pos])) {
            if (is_word) {
                word += sentence[pos];
            } else {
                word = sentence[pos];
                is_word = true;
            }
        } else if (pos == length || sentence[pos] == ' ') {
            if (is_word) {
                if (occured_words.find(word) != occured_words.end()) {
                    return false;
                }
                occured_words.insert(word);
                
                if (nouns.find(word) != nouns.end()) {
                    noun_occured = true;
                    last_adjective = false;
                    score += nouns[word];
                } else if (verbs.find(word) != verbs.end()) {
                    if (verb_occured) {
                        if (occured_words.find(word) != occured_words.end()) {
                            return false;
                        }
                    }
                    if (last_adjective) {
                        return false;
                    }
                    verb_occured = true;
                    score += verbs[word];
                } else if (adjs.find(word) != adjs.end()) {
                    if (last_adjective) {
                        return false;
                    }
                    last_adjective = true;
                    score += adjs[word];
                } else {
                    return false;
                }
                
                ++words;
                is_word = false;
            }
        } else {
            return false;
        }
    }
    
    if (last_adjective) {
        return false;
    }
    if (!noun_occured) {
        return false;
    }
    if (!verb_occured) {
        return false;
    }
    return true;
}

int get_answer(InStream &stream, int left, int right, int test) {
    std::string line = stream.readLine();
    std::set<std::string> occured_words;
    
    if (line.empty()) {
        return -1;
    }
    
    if (!isalpha(line[0])) {
        return -1;
    }
    
    int end = (int)line.length();
    while (line[end - 1] == ' ') {
        --end;
    }
    if (line[end - 1] != '.') {
        return -1;
    }
    
    int pos = 0, words = 0, score = 0;
    
    while (pos != end) {
        int full_stop = (int)line.find('.', pos);
        proceed_sentence(line.substr(pos, full_stop - pos), words, score, occured_words, test);
        pos = full_stop + 1;
        
        if (pos != end && line[pos] != ' ') {
            return -1;
        }
    }
    
    if (words < left || words > right) {
        return -1;
    }
    
    return score;
}

int main(int argc, char **argv) {
    setName("moscow 2015 essay scoring checker");
    registerTestlibCmd(argc, argv);
    
    int tests = inf.readInt(), done = 0;
    
    for (int test = 0; test != tests; ++test) {
        int n = inf.readInt(), v = inf.readInt(), a = inf.readInt();
        for (int i = 0; i != n; ++i) {
            nouns[inf.readWord()] = inf.readInt();
        }
        for (int i = 0; i != v; ++i) {
            verbs[inf.readWord()] = inf.readInt();
        }
        for (int i = 0; i != a; ++i) {
            adjs[inf.readWord()] = inf.readInt();
        }
        
        int left = inf.readInt(), right = inf.readInt();
        
        int ja = get_answer(ans, left, right, -1);
        int pa = get_answer(ouf, left, right, test);
        
        if (ja == -1) {
            quitf(_fail, "%d%s test. Jury WA", test + 1, englishEnding(test + 1).c_str());
        }
        if (pa > ja) {
            quitf(_fail, "%d%s test. Participant got %d points, despite judge answer is %d.", test + 1, englishEnding(test + 1).c_str(),  pa, ja);
        }
        if (pa == ja) {
            ++done;
        }
        
        nouns.clear();
        verbs.clear();
        adjs.clear();
    }
    
    if (done == tests) {
        quitf(_ok, "%d tests are passed.", done);
    } else {
        quitp(floor(((double)done) / ((double)tests) * 40.), "%d tests are passed.", done);
    }
}
